/********************************************************************************
  $Header: /home/farin/cvs/libvideocoding/libvideocoding/dct/fdct_fp_aan.hh,v 1.4 2002/05/09 22:00:31 farin Exp $

    Accurate floating-point implementation of 8x8 DCT.
 ********************************************************************************
    Copyright (C) 2000 Dirk Farin (see the README for complete list of authors)

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#ifndef LIBVIDEOCODING_DCT_FDCT_FP_AAN_HH
#define LIBVIDEOCODING_DCT_FDCT_FP_AAN_HH

#include <libvideocoding/dct/dct.hh>

#define CIRCUMVENT_GCC296_FDCT_BUG 1



namespace videocoding {

  template <class PixelType> class FDCTBlk_AccurateFP : public FDCTBlk8x8<PixelType>
  {
  public:
    FDCTBlk_AccurateFP() { Init_fdct(); }

    void CalcFDCT(DCTBlock_8x8&, const PixelType* in,int in_stride);
    const char* Name() const { return "scalar floating point"; }
  };


  // --------------------------- implementation ----------------------------------


  /* D.Farin: this file has been modified to adapt the interface to the
     DCT classes of the SAMPEG-4 implementation. */


  /* fdctref.c, forward discrete cosine transform, double precision           */

  /* Copyright (C) 1996, MPEG Software Simulation Group. All Rights Reserved. */

  /*
   * Disclaimer of Warranty
   *
   * These software programs are available to the user without any license fee or
   * royalty on an "as is" basis.  The MPEG Software Simulation Group disclaims
   * any and all warranties, whether express, implied, or statuary, including any
   * implied warranties or merchantability or of fitness for a particular
   * purpose.  In no event shall the copyright-holder be liable for any
   * incidental, punitive, or consequential damages of any kind whatsoever
   * arising from the use of these programs.
   *
   * This disclaimer of warranty extends to the user of these programs and user's
   * customers, employees, agents, transferees, successors, and assigns.
   *
   * The MPEG Software Simulation Group does not represent or warrant that the
   * programs furnished hereunder are free of infringement of any third-party
   * patents.
   *
   * Commercial implementations of MPEG-1 and MPEG-2 video, including shareware,
   * are subject to royalty fees to patent holders.  Many of these patents are
   * general enough such that they are unavoidable regardless of implementation
   * design.
   *
   */

#include <math.h>
#include <stdio.h>


#define NC_COS6      0.382683432365089771728459984030399//cos(6*pi/16)

#define NC_R_SQRT2   0.707106781186547524400844362104849// 1/sqrt(2)

#define NC_COS1SQRT2 1.38703984532214746182161919156644 //cos(1*pi/16)*sqrt(2)
#define NC_COS2SQRT2 1.30656296487637652785664317342719 //cos(2*pi/16)*sqrt(2)
#define NC_COS3SQRT2 1.17587560241935871697446710461126 //cos(3*pi/16)*sqrt(2)
#define NC_COS5SQRT2 0.785694958387102181277897367657217//cos(5*pi/16)*sqrt(2)
#define NC_COS6SQRT2 0.541196100146196984399723205366389//cos(6*pi/16)*sqrt(2)
#define NC_COS7SQRT2 0.275899379282943012335957563669373//cos(7*pi/16)*sqrt(2)

  extern void Init_fdct();

  extern const double aansf[8];
  extern double aanscales[8][8];
  extern double costab[8 /* x */][8 /* u */];



  /*
   * Perform a floating point forward DCT on one block of samples.
   */

#ifndef PI
#define PI 3.1415926535
#endif


  template <class PixelType> void FDCTBlk_AccurateFP<PixelType>::CalcFDCT(DCTBlock_8x8& dctblk,
									  const PixelType* in,
									  int in_stride)
									
  {
#if CIRCUMVENT_GCC296_FDCT_BUG
    /*** THIS CODE IS VERY SLOW !!! HOWEVER, AT LEAST GCC 2.96 PRODUCES WRONG CODE
	 FOR THE FAST ALGORITHM BELOW. IF YOU ARE SURE THAT YOUR COMPILER GENERATES
	 CORRECT CODE, YOU CAN SWITCH TO THE CODE BELOW.
    ***/

    for (int u=0;u<8;u++)
      for (int v=0;v<8;v++)
	{
	  double sum=0.0;

	  for (int y=0;y<8;y++)
	    for (int x=0;x<8;x++)
	      {
		int input = in[y*in_stride+x];
		double val = input*costab[x][u]*costab[y][v];
		if (u==0) val *= sqrt(0.5);
		if (v==0) val *= sqrt(0.5);
		sum += val*0.25;
	      }

	  dctblk.coeff[v][u] = (int)(sum+0.5);
	}

#else
    /* Pass 1: process rows. */

    double tmp0, tmp1, tmp2, tmp3, tmp4, tmp5, tmp6, tmp7;
    double tmp10, tmp11, tmp12, tmp13;
    double z1, z2, z3, z4, z5, z11, z13;
    double *dataptr;
    double data[64];

    dataptr = data;
    for (int i = 0; i < 8; i++)
      {
	const PixelType* blkptr = in +i*in_stride;

	tmp0 = blkptr[0] + blkptr[7];
	tmp7 = blkptr[0] - blkptr[7];
	tmp1 = blkptr[1] + blkptr[6];
	tmp6 = blkptr[1] - blkptr[6];
	tmp2 = blkptr[2] + blkptr[5];
	tmp5 = blkptr[2] - blkptr[5];
	tmp3 = blkptr[3] + blkptr[4];
	tmp4 = blkptr[3] - blkptr[4];

	/* Even part */

	tmp10 = tmp0 + tmp3;	/* phase 2 */
	tmp13 = tmp0 - tmp3;
	tmp11 = tmp1 + tmp2;
	tmp12 = tmp1 - tmp2;

	dataptr[0] = tmp10 + tmp11; /* phase 3 */
	dataptr[4] = tmp10 - tmp11;

	z1 = (tmp12 + tmp13) * ((double) NC_R_SQRT2); /* c4 */
	dataptr[2] = tmp13 + z1;	/* phase 5 */
	dataptr[6] = tmp13 - z1;

	/* Odd part */

	tmp10 = tmp4 + tmp5;	/* phase 2 */
	tmp11 = tmp5 + tmp6;
	tmp12 = tmp6 + tmp7;

	/* The rotator is modified from fig 4-8 to avoid extra negations. */
	z5 = (tmp10 - tmp12) * ((double) NC_COS6); /* c6 */
	z2 = ((double) NC_COS6SQRT2) * tmp10 + z5; /* c2-c6 */
	z4 = ((double) NC_COS2SQRT2) * tmp12 + z5; /* c2+c6 */
	z3 = tmp11 * ((double) NC_R_SQRT2); /* c4 */

	z11 = tmp7 + z3;		/* phase 5 */
	z13 = tmp7 - z3;

	dataptr[5] = z13 + z2;	/* phase 6 */
	dataptr[3] = z13 - z2;
	dataptr[1] = z11 + z4;
	dataptr[7] = z11 - z4;

	dataptr += 8;		/* advance pointer to next row */
      }


    /* Pass 2: process columns. */

    dataptr = data;
    for (int i = 0; i < 8; i++)
      {
	tmp0 = dataptr[0] + dataptr[56];
	tmp7 = dataptr[0] - dataptr[56];
	tmp1 = dataptr[8] + dataptr[48];
	tmp6 = dataptr[8] - dataptr[48];
	tmp2 = dataptr[16] + dataptr[40];
	tmp5 = dataptr[16] - dataptr[40];
	tmp3 = dataptr[24] + dataptr[32];
	tmp4 = dataptr[24] - dataptr[32];

	/* Even part */

	tmp10 = tmp0 + tmp3;	/* phase 2 */
	tmp13 = tmp0 - tmp3;
	tmp11 = tmp1 + tmp2;
	tmp12 = tmp1 - tmp2;

	dataptr[0] = tmp10 + tmp11; /* phase 3 */
	dataptr[32] = tmp10 - tmp11;

	z1 = (tmp12 + tmp13) * ((double) NC_R_SQRT2); /* c4 */
	dataptr[16] = tmp13 + z1; /* phase 5 */
	dataptr[48] = tmp13 - z1;

	/* Odd part */

	tmp10 = tmp4 + tmp5;	/* phase 2 */
	tmp11 = tmp5 + tmp6;
	tmp12 = tmp6 + tmp7;

	/* The rotator is modified from fig 4-8 to avoid extra negations. */
	z5 = (tmp10 - tmp12) * ((double) NC_COS6); /* c6 */
	z2 = ((double) NC_COS6SQRT2) * tmp10 + z5; /* c2-c6 */
	z4 = ((double) NC_COS2SQRT2) * tmp12 + z5; /* c2+c6 */
	z3 = tmp11 * ((double) NC_R_SQRT2); /* c4 */

	z11 = tmp7 + z3;		/* phase 5 */
	z13 = tmp7 - z3;

	dataptr[40] = z13 + z2; /* phase 6 */
	dataptr[24] = z13 - z2;
	dataptr[8] = z11 + z4;
	dataptr[56] = z11 - z4;

	dataptr++;			/* advance pointer to next column */
      }

    /* descale */
    int l=0;
    for (int i = 0; i < 8; i++)
      for (int k = 0; k < 8; k++,l++)
	{
	  dctblk.coeff[i][k] = (int16) floor(data[l] * aanscales[i][k] + 0.5);
	}
#endif
  }

}


#endif
