/********************************************************************************
  $Header: /home/farin/cvs/libvideocoding/libvideocoding/mpeg2/syntax/dct.hh,v 1.4 2002/05/12 21:41:47 farin Exp $

    MPEG DCT entropy coding routines.
 ********************************************************************************
    Copyright (C) 2001 Dirk Farin (see the README for complete list of authors)

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#ifndef LIBVIDEOCODING_MPEG2_SYNTAX_DCT_HH
#define LIBVIDEOCODING_MPEG2_SYNTAX_DCT_HH

#include <libvideocoding/datatypes/dct.hh>
#include <libvideocoding/mpeg2/syntax/video.hh>
#include <libvideogfx.hh>

namespace videocoding {
  using namespace videogfx;

  class MP2_DCTBlockEncoder
  {
  public:
    MP2_DCTBlockEncoder();

    /* initialization */

    void SetBitstream(BitBuffer& bb) { d_bs=&bb; }
    void SetParams(bool UseMPEG1,bool UseB15=false,bool UseAlternateScan=false);
    void SetParams(const MP2_PictureHeader& pichdr); // equivalent to method above

    /* usage */

    void Write(const DCTBlock_8x8&,int& predictor,bool isY,bool intra);

    int  IsEscapeCode(int run,int value) const;
    int  QueryCodeLength(int run,int value,bool first_in_intra) const;

  private:
    BitBuffer* d_bs;

    bool d_mpeg1;
    bool d_b15;
    const struct Scan* d_scan;
  };


  class MP2_DCTBlockDecoder
  {
  public:
    MP2_DCTBlockDecoder();

    /* initialization */

    void SetPictureHeader(const MP2_PictureHeader&);

    void EnableOnTheFlyDequantization(bool flag=true) { d_on_the_fly_dequant=flag; }
    void SetQCode(int qcode);

    /* usage */

    int  DecodeIntra(DCTBlock_8x8& blk, BitReader_Fast&, bool lumi);  // returns new DC predictor
    void DecodeInter(DCTBlock_8x8& blk, BitReader_Fast&, bool lumi);

  private:
    const MP2_PictureHeader* d_pichdr;

    bool d_on_the_fly_dequant;
    int  d_qscale;

    const int* d_scan;
  };

}

#endif
